#!/bin/bash -e
#
# prepare-raspbian-sd:bin/prepare-raspbian-config-txt-api.sh
#
declare -gr _PREPARE_RASPBIAN_CONFIG_DIR_="${HOME}/.config/prepare-raspbian-sd"
declare -gr _PREPARE_RASPBIAN_CONFIG_VARS_=(
'DEFAULT_SD_DEV_NAME:STRING'
'GPU_MEM:INTEGER'
'RASPBIAN_DEFAULT_BACKUP_DIRECTORY:STRING'
'RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY:STRING'
'RASPBIAN_DEFAULT_IMG:STRING'
'RASPBIAN_PI_GROUP_ID:INTEGER'
'RASPBIAN_PI_USER_ID:INTEGER'
'SSH_AUTHORIZED_KEYS:STRING'
)

function prepare_raspbian_show_list {
  local entry=

  for entry in "${_PREPARE_RASPBIAN_CONFIG_VARS_[@]}" ; do
    cut -d ':' -f1 <<<"${entry}"
  done
}

function prepare_raspbian_show_list_all {
  local var=

  while read -r var ; do
    prepare_raspbian_check_and_get_value "${var}"
  done < <( prepare_raspbian_show_list )
}

function prepare_raspbian_config_is_var_name_valide {
  local -r var_name="$1"
  local    var=

  while read -r var ; do
   if [ "${var_name}" = "${var}" ] ; then
    return 0
   fi
  done < <( prepare_raspbian_show_list )

  return 1
}

function prepare_raspbian_config_check_var_name {
  local -r var_name="$1"

  if ! prepare_raspbian_config_is_var_name_valide "${var_name}" ; then
    echofatal "Wrong config entry. What is '${var_name}' ?"
    return 1
  fi

  return 0
}

function ensure_raspbian_config_ready {
  if [ ! -d "${_PREPARE_RASPBIAN_CONFIG_DIR_}" ] ; then
    mkdir -vp "${_PREPARE_RASPBIAN_CONFIG_DIR_}"
  fi

  exit_if_directory_not_exists "${_PREPARE_RASPBIAN_CONFIG_DIR_}"
}

function prepare_raspbian_initialise_if_needed {
  local -r var_name="$1"
  local -r default_value="${2:-}"

  ensure_raspbian_config_ready

  if ! is_variable_define_and_not_empty_safe "${var_name}" ; then
    if [ -f "${_PREPARE_RASPBIAN_CONFIG_DIR_}/${var_name}" ] ; then
      declare "${var_name}"="$( <"${_PREPARE_RASPBIAN_CONFIG_DIR_}/${var_name}" )"
    fi

    if ! is_variable_define_and_not_empty_safe "${var_name}" ; then
      if [ -n "${default_value}" ] ; then
        echowarn "${var_name} is not define" >&2
        declare "${var_name}"="${default_value}"
        echowarn "Use ${var_name}=${!var_name}" >&2
      fi
    fi

    if is_variable_define_and_not_empty_safe "${var_name}" ; then
      declare -gr "${var_name}=${!var_name}"
      # shellcheck disable=SC2163
      export "${var_name}"
    fi
  fi
}

function _prepare_raspbian_config_set_ {
  local -r var_name="$1"
  local -r var_value="$2"

  ensure_raspbian_config_ready

  if [ -n "${var_value}" ] ; then
    echo "${var_value}" >"${_PREPARE_RASPBIAN_CONFIG_DIR_}/${var_name}" || exit 1
    echoinfo "'${var_name}' is set to value: '${var_value}'" >&2
  else
    if [ -f "${_PREPARE_RASPBIAN_CONFIG_DIR_}/${var_name}" ] ; then
      rm -v "${_PREPARE_RASPBIAN_CONFIG_DIR_}/${var_name}" >&2 || exit 1
    fi
    echoinfo "'${var_name}' is removed." >&2
  fi
}

function prepare_raspbian_check_and_set_value_string {
  local -r var_name="$1"
  local -r var_value="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_ "${var_name}" "${var_value}"
  else
    return 1
  fi
}

function prepare_raspbian_check_and_set_value_integer {
  local -r var_name="$1"
  local -r var_value="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_ "${var_name}" "${var_value}"
  else
    return 1
  fi
}

function _prepare_raspbian_config_set_link_ {
  local -r var_name="$1"
  local -r var_link="$2"

  ensure_raspbian_config_ready

  if [ -n "${var_link}" ] ; then
    local -r var_fullname="${_PREPARE_RASPBIAN_CONFIG_DIR_}/${var_name}"

    ln -sf "${var_link}" "${var_fullname}" || exit 1
    echoinfo "'${var_name}' is link to value: '$( cat "${var_fullname}" )'" >&2
  else
    if [ -f "${_PREPARE_RASPBIAN_CONFIG_DIR_}/${var_name}" ] ; then
      rm -v "${_PREPARE_RASPBIAN_CONFIG_DIR_}/${var_name}" >&2 || exit 1
    fi
    echoinfo "'${var_name}' is removed." >&2
  fi
}

function prepare_raspbian_check_and_set_value_link {
  local -r var_name="$1"
  local -r var_link="$2"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    _prepare_raspbian_config_set_link_ "${var_name}" "${var_link}"
  else
    return 1
  fi
}

function prepare_raspbian_check_and_set_value {
  local -r var_type="$1"
  local -r var_name="$2"
  local -r var_value="$3"

  case "${var_type}" in
    'INTEGER')
      if ! is_positive_integer "${var_value}" ; then
        echofatal "Value must be a postive integer: '${var_value}'"
        exit 1
      fi
      prepare_raspbian_check_and_set_value_integer "${var_name}" "${var_value}"
      ;;

    'LINK')
      if [ ! -f "${var_value}" ] ; then
        echofatal "File not found: '${var_value}'"
        exit 1
      fi
      prepare_raspbian_check_and_set_value_link "${var_name}" "${var_value}"
      ;;

    'STRING')
      prepare_raspbian_check_and_set_value_string "${var_name}" "${var_value}"
      ;;

    *)
      echofatal "Unsupported variable type: '${var_type}'"
      exit 1
      ;;
  esac
}

function prepare_raspbian_check_and_get_value {
  local -r var_name="$1"

  if prepare_raspbian_config_check_var_name "${var_name}" ; then
    prepare_raspbian_initialise_if_needed "${var_name}"

    if is_variable_define_and_not_empty_safe "${var_name}" ; then
      echo "${var_name}=${!var_name}" >&2
      echo "${!var_name}"
      return 0
    else
      echo "${var_name} not define." >&2
      return 1
    fi
  else
    return 1
  fi
}

