#!/bin/bash -e
#
# prepare-raspbian-sd:bin/prepare-raspbian-customize-api.sh
#
declare -gri RASPBIAN_PI_USER_ID_DEFAULT=1000
declare -gri RASPBIAN_PI_GROUP_ID_DEFAULT=1000

declare -gr FILES_TO_VERIFY_ON_BOOT_PARTITION='
config.txt
kernel.img
kernel7.img
start.elf
start_x.elf
'

declare -gr FILES_TO_VERIFY_ON_ROOTFS_PARTITION='
home/pi/.bashrc
home/pi/.profile
usr/bin/raspi-config
'

function customize_sd {
  local boot_partition_home=
  local rootfs_partition_home=

  # Ensure boot partition is has expected
  boot_partition_home="$( get_partition_home_by_label_with_timeout 'boot' )" || exit 1
  echoinfo "boot_partition_home=${boot_partition_home}"

  # Ensure rootfs partition is has expected
  rootfs_partition_home="$( get_partition_home_by_label_with_timeout 'rootfs' )" || exit 1
  echoinfo "rootfs_partition_home=${rootfs_partition_home}"

  verify_sd_content "${boot_partition_home}" "${rootfs_partition_home}"

  customize_enable_ssh "${boot_partition_home}" # TODO: should be optional
  customize_memory_split "${boot_partition_home}"
  customize_pi_user "${rootfs_partition_home}/home/pi"

  sync

  #
  # umount if it looks okay
  #
  local ssh_file="${boot_partition_home}/ssh"

  if [ -f "${ssh_file}" ] ; then
    echoinfo "Umounting '${boot_partition_home}'"
    sudo umount "${boot_partition_home}" || exit 1
    echookay "${boot_partition_home} umounted."

    echoinfo "Umounting '${rootfs_partition_home}'"
    sudo umount "${rootfs_partition_home}" || exit 1
    echookay "${rootfs_partition_home} umounted."
  else
    echowarn "ssh_file='${ssh_file}' not found."
  fi
}

function customize_enable_ssh {
  local -r boot_partition_home="$1"

  if [ ! -d "${boot_partition_home}" ] ; then
    echofatal "Cannot find directory boot_partition_home='${boot_partition_home}'"
    exit 1
  fi

  echoinfo 'Enable SSH.'

  local -r ssh_file="${boot_partition_home}/ssh"

  if [ -f "${ssh_file}" ] ; then
    echoinfo "File '${ssh_file}' already exists."
  else
    touch "${ssh_file}"
  fi
}

function customize_memory_split {
  local -r boot_partition_home="$1"
  local -r config_txt="${boot_partition_home}/config.txt"

  prepare_raspbian_initialise_if_needed GPU_MEM 0

  if [ "${GPU_MEM}" = '0' ] ; then
    echowarn "customize_memory_split(): GPU_MEM is set to ${GPU_MEM} - Ignore memory split configuration"
    return 0
  fi

  if [ ! -d "${boot_partition_home}" ] ; then
    echofatal "customize_memory_split(): Cannot find directory boot_partition_home='${boot_partition_home}'"
    exit 1
  fi

  if [ ! -f "${config_txt}" ] ; then
    echofatal "customize_memory_split(): Cannot find file config_txt='${config_txt}'"
    exit 1
  fi

  if grep --quiet '^gpu_mem=' "${config_txt}" ; then
    echowarn "customize_memory_split(): Memory split is already define - '${config_txt}'"
    grep '^gpu_mem=' "${config_txt}" >&2
    return 0
  fi

  echoinfo "customize_memory_split(): Define memory split to minimum ('${GPU_MEM}' MB) - '${config_txt}'"
  cat <<EOF >> "${config_txt}"
# refs:
#   - https://elinux.org/RPiconfig
#   - https://www.raspberrypi.org/documentation/configuration/config-txt/memory.md
# gpu_mem=16 is the minimum possible value.
# maximum value is 192, 448, or 944, depending on whether you are using a 256M, 512MB, 1024MB or more Pi. The default value is 64.
# gpu_mem minimum GPU memory for camera use: gpu_mem=128
gpu_mem=${GPU_MEM}
EOF
}

function customize_pi_user {
  local -r home_pi="$1"

  prepare_raspbian_initialise_if_needed RASPBIAN_PI_USER_ID  "${RASPBIAN_PI_USER_ID_DEFAULT}"
  prepare_raspbian_initialise_if_needed RASPBIAN_PI_GROUP_ID "${RASPBIAN_PI_GROUP_ID_DEFAULT}"

  if [ ! -d "${home_pi}" ] ; then
    echofatal "Cannot find directory: /home/pi='${home_pi}'"
    exit 1
  fi

  # Create bcm workspace directory (TODO: should be optional)
  local -r bcm_workspace="${home_pi}/.bcm-workspace"

  if [ -d "${bcm_workspace}" ] ; then
    echoinfo "Folder '${bcm_workspace}' already exists."
  else
    sudo mkdir -v "${bcm_workspace}"
    # Set user right to 'pi' user and 'pi' group
    sudo chown "${RASPBIAN_PI_USER_ID}:${RASPBIAN_PI_GROUP_ID}" "${bcm_workspace}" || exit 1
  fi

  # Configure ssh (TODO: should be optional)
  local -r ssh_folder="${home_pi}/.ssh"

  if [ -d "${ssh_folder}" ] ; then
    echoinfo "Folder '${ssh_folder}' already exists."
  else
    sudo mkdir -v "${ssh_folder}"
    # Set user right to 'pi' user and 'pi' group
    sudo chown "${RASPBIAN_PI_USER_ID}:${RASPBIAN_PI_GROUP_ID}" "${ssh_folder}" || exit 1
    sudo chmod 700                                              "${ssh_folder}" || exit 1
  fi

  prepare_raspbian_initialise_if_needed SSH_AUTHORIZED_KEYS ''

  if ! is_variable_define_and_not_empty_safe 'SSH_AUTHORIZED_KEYS' ; then
    echowarn 'SSH_AUTHORIZED_KEYS not define (or empty) - you can fix this in file define by RASPBIAN_SSH_CONFIG'

    if is_variable_define_and_not_empty_safe 'RASPBIAN_SSH_CONFIG' ; then
      echoerror "Check content of '${RASPBIAN_SSH_CONFIG}'"
    fi
  else
    if sudo [ -f "${ssh_folder}/authorized_keys" ] ; then
      echowarn "File '${ssh_folder}/authorized_keys' already exists (nothing will be changed)."
    else
      cat <<EOF | sudo tee "${ssh_folder}/authorized_keys"
${SSH_AUTHORIZED_KEYS}
EOF
      # Set user right to 'pi' user and 'pi' group
      sudo chown "${RASPBIAN_PI_USER_ID}:${RASPBIAN_PI_GROUP_ID}" "${ssh_folder}/authorized_keys" || exit 1
      echoinfo "File '${ssh_folder}/authorized_keys' created."
    fi
  fi
}

function get_partition_home_by_label_with_timeout {
  local -r label="$1"
  local    partition_home=

  for count in $(seq 10 -1 1) ; do
    if partition_home="$( get_partition_home_by_label "${label}" )" ; then
      echo "${partition_home}"
      return 0
    fi
    echowarn "'${label}' partition not found. Will try again ${count} times."
    sleep 2
  done

  echofatal "'${label}' partition not found."
  exit 1
}

function get_partition_home_by_label {
  local -r label="$1"
  local    partition_home=

  partition_home="$(
    mount --show-labels | grep "\\[${label}\\]" | cut -d ' ' -f3
    )"

  if [ -z "${partition_home}" ] ; then
    return 1
  else
    echoinfo "${partition_home}" >&2
    echo "${partition_home}"
  fi
}

function verify_sd_content {
  local -r boot_partition_home="$1"
  local -r rootfs_partition_home="$2"

  if [ ! -d "${boot_partition_home}" ] ; then
    echoerror "Cannot find directory: boot_partition_home='${boot_partition_home}'"
    exit 1
  fi

  for file in ${FILES_TO_VERIFY_ON_BOOT_PARTITION} ; do
    exit_if_file_not_exists "${boot_partition_home}/${file}"
  done

  if [ ! -d "${rootfs_partition_home}" ] ; then
    echoerror "Cannot find directory: rootfs_partition_home='${rootfs_partition_home}'"
    exit 1
  fi

  for file in ${FILES_TO_VERIFY_ON_ROOTFS_PARTITION} ; do
    exit_if_file_not_exists "${rootfs_partition_home}/${file}"
  done
}
