#!/bin/bash
#
# prepare-raspbian-sd:bin/prepare-raspbian-sd-config.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -gr PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/prepare-raspbian-config-json-api.sh" || exit 1

function usage {
  cat <<EOF
Allow to configure prepare-raspbian-sd default values.

Usage:

   $( basename "$0" ) --help
   $( basename "$0" ) --get VARNAME
   $( basename "$0" ) --list
   $( basename "$0" ) --list-all
   $( basename "$0" ) --set-integer VARNAME VALUE
   $( basename "$0" ) --set-link VARNAME LINK_TARGET
   $( basename "$0" ) [--set] VARNAME VALUE

Actions:
  -h, --help                Display help message and exit.
      --get VARNAME         Show given variable content
  -l, --list                Display valid variables name's
  -A, --all, --list-all     Show all define variables in JSON format.
      --set-integer VARNAME VALUE
                            Set given VALUE to variable VARNAME. VALUE must be an integer.
      --set-link VARNAME LINK_TARGET
                            Set given VALUE to the content of given LINK_TARGET.
                            Related VALUE will be content of this file.
      --set VARNAME VALUE   Set given VALUE to variable VARNAME.

EOF
}

function test_action {
  if [ -n "${action}" ] ; then
    echofatal "Action already set to '${action}'"
    usage >&2
    exit 1
  fi
}

function main {
  local action=
  local param=
  local param_name='_WRONG_NAME_'
  local param_value='_WRONG_VALUE_'

  if [ $# -eq 0 ] ; then
    echofatal 'Parameters missing.'
    usage >&2
    exit 1
  fi

  while [ $# -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '-l'|'--list')
        test_action && action='--list'
        ;;

      '-A'|'--all'|'--list-all')
        test_action && action='--list-all'
        ;;

      '--get')
        test_action && action='--get'
        param_name="$( P "$@" )" && shift
        ;;

      '--set')
        test_action && action='--set'
        param_name="$( P "$@" )" && shift
        param_value="$( P "$@" )" && shift
        ;;

      '--set-integer')
        test_action && action='--set-integer'
        param_name="$( P "$@" )" && shift
        param_value="$( P "$@" )" && shift
        ;;

      '--set-link')
        test_action && action='--set-link'
        param_name="$( P "$@" )" && shift
        param_value="$( P "$@" )" && shift
        ;;

      *)
        test_action && action='--set'
        param_name="${param}"
        param_value="$( P "$@" )" && shift
        ;;
    esac
  done

  case "${action}" in
    '--list')
      prepare_raspbian_show_list || return $?
      ;;

    '--list-all')
      prepare_raspbian_show_list_all || return $?
      ;;

    '--set')
      prepare_raspbian_check_and_set_value 'STRING' "${param_name}" "${param_value}" || return $?
      ;;

    '--set-integer')
      prepare_raspbian_check_and_set_value 'INTEGER' "${param_name}" "${param_value}" || return $?
      ;;

    '--set-link')
      prepare_raspbian_check_and_set_value 'LINK' "${param_name}" "${param_value}" || return $?
      ;;

    '--get')
      prepare_raspbian_check_and_get_value "${param_name}" || return $?
      ;;

    *)
      echofatal "Internal error: action='${action}'"
      exit 1
      ;;
  esac
}

main "$@"
