#!/bin/bash
#
# prepare-raspbian-sd:bin/prepare-raspbian-sd.sh
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -gr PROJECT_HOME

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/prepare-raspbian-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/prepare-raspbian-config-json-api.sh" || exit 1
# shellcheck disable=SC1090
source "${PROJECT_HOME}/bin/prepare-raspbian-customize-api.sh" || exit 1

function usage {
  cat <<EOF
Usage: $0

   $( basename "$0" ) --help
   $( basename "$0" ) ACTION [OPTIONS]...

Puts the image of the Raspberry Pi (formerly Raspbian) operating system on an SD card
and allows it to be configured. Also allows you to install images of which you know the
URL. Also allows you to create an image from an SD card.

Actions

  -h,       --help          Display help message and exit.
  -I FILE,  --put-image FILE
                            Initialize the SD card using the FILE image file.
  -D,       --put-default-image
                            Initialize the SD card using the defined image by RASPBIAN_DEFAULT_IMG:
                            '$( print_variable RASPBIAN_DEFAULT_IMG )'
  -L,       --put-last-image
                            Initialize the SD card using the latest available version of the Raspbian image.
                            Download the image from the Raspbian website if necessary.
            --configure     Apply the user configuration on the SD card.
  -F,       --full-clone    Create an SD card image in the directory defined by
                            RASPBIAN_DEFAULT_BACKUP_DIRECTORY (a name file will be generated from the date).
                            RASPBIAN_DEFAULT_BACKUP_DIRECTORY:
                            '$( print_variable RASPBIAN_DEFAULT_BACKUP_DIRECTORY )'
  -T FILE,  --full-clone-to FILE
                            Create an image of the SD card using given name. FILE could be a relative or
                            absolute filename. Fail if path does not exists or file already exists.
  --compute-sd-dev-name     Locate the SD card and display the names of associated devices, then exit.
  --dry-run                 Show used SD devices, then exit.

Options

  --arch ARCH           Define machine hardware name (optional). If not set, or empty, use "Raspbian"
                        as source, if set use "Raspberry Pi OS" as source). ARCH could be an empty string,
                        armhf or arm64.
                          --arch '' : Download last available version of raspbian
                          --arch armhf: Download last 32 bits version of "Raspberry Pi OS"
                          --arch arm64: Download last 64 bits version of "Raspberry Pi OS"
  --desktop             Same as --full (deprecated)
  --dev-name DEVNAME    Forces the name of the device to be used for the SD card.
                        Default value: '$( print_variable DEFAULT_SD_DEV_NAME )'
  --download-to DIR     Upload a new image if necessary in the given directory. If the option value is
                        omitted the value defined by RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY will be used.
                        RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY:
                        '$( print_variable RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY )'
  --expected-img IMG    Expected image file name (no path). Mandatory with action --url does not download
                        the remote file if a local file is present.
                        (If use with --unzip must be unzip name of file)
  --full                Download full version (the desktop version) of Raspberry Pi OS instead of the
                        light version (which is server version).
  --gui                 Same as --full (deprecated)
  --unzip               Unzip file after downloading, and use unzipped file as image file
                        (gzip and unzip supported)


EOF

  if [ "${1:-}" = '--help' ] ; then
    cat <<EOF
Security:
  To write to SD (except for the action --configure), the tool checks that the SD card is NOT mounted/
  You must be running umount (or using the interface graphic)
    sudo umount /dev/<DEVNAME>

Samples:

  prepare-raspbian-sd -L --arch armhf # 32 bits version
  prepare-raspbian-sd -L --arch arm64 # 64 bits version (warn: 64 bits is not yet ready, see README.md)

    Initialize tour SD card with last Rasperry Pi OS image (SD must be plugin but not mounted)

  prepare-raspbian-sd -L

    Initialize your SD card with last Raspbian image (SD must be plugin but not mounted)

  prepare-raspbian-sd -T ~/my-sd-backup.img

    Create an image of your SD named ~/my-sd-backup.img

  prepare-raspbian-sd -F

    Create an image of your SD, name is generated and use folder define by RASPBIAN_DEFAULT_BACKUP_DIRECTORY
    According to your configuration:
      '$( print_variable RASPBIAN_DEFAULT_BACKUP_DIRECTORY )'

EOF
  fi
}

function get_last_img_version {
  local -a options=()

  [ -n "${param_full}" ] && options+=("${param_full}")
  [ -n "${param_arch}" ] && options+=(--arch "${param_arch}")

  local output=
  output="$(
    "${PROJECT_HOME}/bin/prepare-raspbian-sd-download-image.sh" --unzip --download-to "${download_to}" "${options[@]}"
    )" || exit 1

  grep 'LAST_IMG_FILE:' <<<"${output}" | cut -d ':' -f2
}

function download_img_file {
  local unzip=

  if [ "${param_unzip}" = 'true' ] ; then
    unzip='--unzip'
  fi
  if [ -z "${param_url}" ] ; then
    echofatal 'Value for --url is missing.'
    exit 1
  fi
  if [ -z "${param_expected_img}" ] ; then
    echofatal 'Value for --expected-img is missing.'
    exit 1
  fi

   cat <<EOF >&2
  "${PROJECT_HOME}/bin/prepare-raspbian-sd-download-image.sh" ${unzip} \
        --download-to "${download_to}" \
        --url "${param_url}" \
        --expected-img "${param_expected_img}"
EOF
  "${PROJECT_HOME}/bin/prepare-raspbian-sd-download-image.sh" ${unzip} \
        --download-to "${download_to}" \
        --url "${param_url}" \
        --expected-img "${param_expected_img}" |
    grep 'LAST_IMG_FILE:' |
    cut -d ':' -f2
}

function get_default_backup_file {
  echo "${RASPBIAN_DEFAULT_BACKUP_DIRECTORY}/$( date +'%Y-%m-%d.%H-%M' )-SD.img"
}

function erase_sd_card_with_image {
  local -r sd_full_dev_name="$1"
  local -r image_file="$2"

  if [ ! -f "${image_file}" ] ; then
    echoerror "Image file '${image_file}' not found."
    exit 1
  fi

  if mount | grep "${sd_full_dev_name}" ; then
    local dev=
    echofatal "SD Card '${sd_full_dev_name}' is mounted, you need to run:"

    while read -r dev ; do
      echo "sudo umount ${dev}"
    done < <( mount | grep "${sd_full_dev_name}" | cut -d ' ' -f1 | sort )
    exit 1
  fi

  echoinfo "Prepare SD based on: '${image_file}'"

  # Cancel ?
  if ! whiptail --yesno --defaultno "Would you like to erase content of '${sd_full_dev_name}' ?" 7 60 2 ; then # not yes
    echoinfo "Cancel by user."
    exit 1
  fi

  date
  cat <<EOF
  sudo dd bs=4M if="${image_file}" of="${sd_full_dev_name}" status=progress conv=fsync
EOF
  sudo dd bs=4M if="${image_file}" of="${sd_full_dev_name}" status=progress conv=fsync
  sync
  date
}

function create_sd_complete_image {
  local -r sd_full_dev_name="$1"
  local -r image_file="$2"

  if [ -f "${image_file}" ] ; then
    echoerror "Image file '${image_file}' already exists."
    exit 1
  fi

  if mount | grep "${sd_full_dev_name}" ; then
    local dev=
    echofatal "SD Card '${sd_full_dev_name}' is mounted, you need to run:"

    while read -r dev ; do
      echo "sudo umount ${dev}"
    done < <( mount | grep "${sd_full_dev_name}" | cut -d ' ' -f1 | sort )
    exit 1
  fi

  date
  cat <<EOF
  sudo dd bs=4M if="${sd_full_dev_name}" of="${image_file}" status=progress conv=fsync
EOF
  sudo dd bs=4M if="${sd_full_dev_name}" of="${image_file}" status=progress conv=fsync
  sync
  date

  local -r currentuser="${USER}"

  if [ -f "${image_file}" ] ; then
    sudo chown -v "${currentuser}:${currentuser}" "${image_file}"
  fi

  echoinfo "Image file '${image_file}' created."
  echoinfo "Real path: '$( realpath "${image_file}" )'"
}

function verify_sd {
  local -r sd_dev_name_only="$1"

  if [ -f "/sys/block/${sd_dev_name_only}/device/type" ] ; then
    read -r is_sd < "/sys/block/${sd_dev_name_only}/device/type"

    if [ "${is_sd}" != 'SD' ] ; then
      echofatal "Content of '/sys/block/${sd_dev_name_only}/device/type' is not 'SD' - found '${is_sd}'"
      exit 1
    fi
  else
    echofatal "File not found: '/sys/block/${sd_dev_name_only}/device/type'"
    exit 1
  fi
}

function get_SD_device {
  (
  for BDEV in /sys/block/* ; do
    if [ -f "${BDEV}/device/type" ] ; then
      echo "$( basename "${BDEV}" ):$( cat "${BDEV}/device/type" ) "
     fi
  done
  ) | grep ':SD' | cut -d ':' -f1
}

function get_SD_device_safe {
  local sd_dev_name=

  sd_dev_name="$( get_SD_device )"

  if [ -z "${sd_dev_name}" ] ; then
     echofatal 'No SD devices found. Do you have a connected SD card?'
  elif [ "$( wc -w <<<"${sd_dev_name}" )" != 1 ] ; then
    echofatal "Computed device for SD give more than one value '${sd_dev_name}'"
    echofatal 'Use must use --dev-name option'
  else
    echo "${sd_dev_name}"
  fi
}

function init_SD_DEV_NAME {
  if [ -n "${force_sd_dev_name}" ] ; then
    declare -gr SD_DEV_NAME="${force_sd_dev_name}"

    echoinfo "Fixed SD_DEV_NAME='${SD_DEV_NAME}'"
  elif [ "${compute_sd_dev_name}" = 'true' ] || ! is_variable_define_and_not_empty_safe DEFAULT_SD_DEV_NAME; then
    SD_DEV_NAME="$( get_SD_device_safe )"
    declare -gr SD_DEV_NAME

    if [ -z "${SD_DEV_NAME}" ] ; then
      echofatal 'Cannot compute SD_DEV_NAME'
      echofatal 'Use must use --dev-name option'
      exit 1
    else
      local -r sd_full_dev_name="/dev/${SD_DEV_NAME}"

      echoinfo "Computed SD_DEV_NAME='${SD_DEV_NAME}'" >&2
      ls -l --color "${sd_full_dev_name}"* >&2
      mount | grep "${sd_full_dev_name}" | cut -d ' ' -f1 | sort
    fi

  else
    declare -gr SD_DEV_NAME="${DEFAULT_SD_DEV_NAME}"

    echoinfo "Default SD_DEV_NAME='${SD_DEV_NAME}'"
  fi
}

function main {
  local action=
  local compute_sd_dev_name='false'
  local download_to=
  local force_sd_dev_name=
  local param=
  local param_arch=
  local param_expected_img=
  local param_full=
  local param_image_file=
  local param_unzip=
  local param_url=

  exit_if_command_is_missing \
    curl \
    dd \
    gzip \
    mount \
    umount \
    unzip \
    whiptail

  while [ $# -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage --help
        exit 0
        ;;

      '-D'|'--put-default-image')
        action='WRITE_ON_SD'
        exit_if_variable_not_define_or_empty RASPBIAN_DEFAULT_IMG
        param_image_file="${RASPBIAN_DEFAULT_IMG}"
        ;;

      '-I'|'--put-image')
        action='WRITE_ON_SD'
        param_image_file="$( P "$@" )" && shift
        ;;

      '-L'|'--put-last-image')
        action='WRITE_ON_SD'
        param_image_file=':LAST:'
        ;;

      '--url')
        action='WRITE_ON_SD'
        param_image_file=':URL:'
        param_url="$( P "$@" )" && shift
        ;;

      '--configure')
        action='CONFIGURE'
        ;;

      '-F'|'--full-clone')
        action='READ_FROM_SD'
        exit_if_variable_not_define_or_empty RASPBIAN_DEFAULT_BACKUP_DIRECTORY
        param_image_file="$( get_default_backup_file )"
        ;;

      '-T'|'--full-clone-to')
        action='READ_FROM_SD'
        param_image_file="$( P "$@" )" && shift
        ;;

      '--dry-run')
        action='DRYRUN'
        ;;

      '--arch')
        param_arch="$( P "$@" )" && shift
        ;;

      '--download-to')
        download_to="$( P "$@" )" && shift
        ;;

      '--desktop'|'--full'|'--gui')
        param_full='--full'
        ;;

      '--dev-name')
        force_sd_dev_name="$( P "$@" )" && shift
        ;;

      '--compute-sd-dev-name')
        action='DRYRUN'
        compute_sd_dev_name='true'
        ;;

      '--expected-img')
        param_expected_img="$( P "$@" )" && shift
        ;;

      '--unzip')
        param_unzip='true'
        ;;

      *)
        # unknown option
        echoerror "Undefine parameter '${param}'."
        usage >&2
        exit 1
        ;;
    esac
  done

  if [ -z "${action}" ] ; then
    echoerror 'Action si missing.'
    usage >&2
    exit 1
  fi

  if [ -z "${download_to}" ] ; then
    exit_if_variable_not_define_or_empty RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY
    download_to="${RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY}"
  fi
  if [ "${param_image_file}" = ':LAST:' ] ; then
    param_image_file="$( get_last_img_version )"

    if [ -z "${param_image_file}" ] ; then
      echofatal 'Cannot get last image'
      exit 1
    fi
  elif [ "${param_image_file}" = ':URL:' ] ; then
    param_image_file="$( download_img_file )"

    if [ -z "${param_image_file}" ] ; then
      echofatal "Cannot get '${param_image_file}'"
      exit 1
    fi
  fi

  init_SD_DEV_NAME

  local -r sd_dev_name_only="${SD_DEV_NAME}"
  local -r sd_full_dev_name="/dev/${SD_DEV_NAME}"

  verify_sd "${sd_dev_name_only}"

  case "${action}" in
    'CONFIGURE')
      customize_sd
      ;;

    'WRITE_ON_SD')
      erase_sd_card_with_image "${sd_full_dev_name}" "${param_image_file}"
      ;;

    'READ_FROM_SD')
      create_sd_complete_image "${sd_full_dev_name}" "${param_image_file}"
      ;;

    'DRYRUN')
      ;;

    *)
      echofatal "Action not valid (${action}')"
      usage >&2
      exit 1
      ;;
  esac
}

prepare_raspbian_initialise_if_needed DEFAULT_SD_DEV_NAME ''
prepare_raspbian_initialise_if_needed RASPBIAN_DEFAULT_BACKUP_DIRECTORY "$( xdg-user-dir DOCUMENTS )"
prepare_raspbian_initialise_if_needed RASPBIAN_DEFAULT_DOWNLOAD_IMAGE_DIRECTORY "$( xdg-user-dir DOWNLOAD )"
prepare_raspbian_initialise_if_needed RASPBIAN_DEFAULT_IMG ''

main "$@"

